<?php
/**
 * Read ebay auction page html code for an auction
 *
 * @package    AuctionHTML
 * @author     Knut Kohl <knutkohl@users.sourceforge.net>
 * @copyright  2007-2009 Knut Kohl
 * @license    http://www.gnu.org/licenses/gpl.txt GNU General Public License
 * @version    1.1.2
 * @since      File available since Release 2.0.0
 */

/**
 * Read ebay auction page html code for an auction
 *
 * @package    AuctionHTML
 * @author     Knut Kohl <knutkohl@users.sourceforge.net>
 * @copyright  2007-2009 Knut Kohl
 * @license    http://www.gnu.org/licenses/gpl.txt GNU General Public License
 * @version    1.0.0
 */
abstract class AuctionHTML {

  /**
   *
   */
  public static $MinSize = 1000;

  /**
   *
   */
  public static $Retry = 3;

  /**
   *
   */
  public static function clearBuffer( $item='*', $urlId='*' ) {
    $files = sprintf('"%s/"%s.%s".html"', TEMPDIR, $item, $urlId);
    if (Exec::getInstance()->Remove($files, $res)) Messages::addInfo($res);
  }

  /**
   *
   */
  public static function &getHTML( $item, $urls, &$err, $urlId='default' ) {
    if (self::$FirstCall) {
      // remove old HTMLs only ONCE each script run
      self::clearBuffer();
      self::$FirstCall = FALSE;
    }

    // check for valid end existing url id
    if (empty($urls[$urlId])) $urlId = 'default';

    $file = sprintf('%s/%s.%s.html', TEMPDIR, $item, $urlId);

    // >> Debug
    DebugStack::Info(sprintf('AuctionHTML::getHTML : Read auction HTML for item "%s", Id "%s"', $item, $urlId));
    // << Debug

    if (file_exists($file)) {
      // >> Debug
      DebugStack::Info('AuctionHTML::getHTML : Found cached file = '.$file);
      // << Debug
      $html = file_get_contents($file);
    } else {
      $url = sprintf($urls[$urlId], $item);
      // >> Debug
      DebugStack::Info('AuctionHTML::getHTML : Read from ebay');
      DebugStack::Info('URL: '.$url);
      DebugStack::Info('Cache file: '.$file);
      // << Debug

      $retry = self::$Retry;
      do {
        $html = HTMLpage::get($url, $err);
        if ($err) {
          Messages::addError($err);
        } else {
          // remove some uninteresting stuff
          $html = preg_replace('~<script.*?</script>~si', '', $html);
          $html = preg_replace('~<noscript>.*?</noscript>~si', '', $html);
          $html = "<!-- $url -->\n" . $html;
          // save to cache file
          File::write($file, $html);
          // save content to file for later debugging
          File::write(str_replace('.html', '-'.date('Ymd-His').'.html', $file), $html);
        }
      } while ($retry-- >= 0 AND strlen($html) < self::$MinSize);
    }
    return $html;
  }

  // -------------------------------------------------------------------------
  // PRIVATE
  // -------------------------------------------------------------------------

  /**
   *
   */
  private static $FirstCall = TRUE;

  /**
   *
   */
  private final function __construct() {}

}