<?php
/**
 * Event handling
 *
 * @package Event
 */

/**
 * Events handling class
 */
abstract class Event implements EventI {

  // >> Debug
  /**
   * Shorten debugging parameter to this length
   *
   * @var int
   */
  public static $DBGLEN = 100;
  // << Debug

  /**
   *
   */
  public static function getData() {
    return self::$Data;
  }

  /**
   *
   */
  public static function setData( $data ) {
    self::$Data = $data;
  }

  /**
   * Naming of event handling methods: <Event>Notification()
   */
  public static $Notification = 'Notification';

  /**
   *
   */
  public static function Attach( EventHandlerI $handler, $position=0 ) {
    /// DebugStack::StartTimer(__METHOD__);
    // make sure not overwrite an function, find next free position
    while (isset(self::$EventHandlers[$position])) $position++;

    self::$EventHandlers[$position] = $handler;
    ksort(self::$EventHandlers);

    self::$HandlerMethods[$position] = array();
    $notification = strtolower(self::$Notification);
    foreach (get_class_methods($handler) as $method) {
      $method = strtolower($method);
      if (strpos($method, $notification)) self::$HandlerMethods[$position][$method] = TRUE;
    }
    /// DebugStack::Info('Attached event handlers: '.count(self::$EventHandlers));
    /// DebugStack::StopTimer(__METHOD__);
  }

  /**
   *
   */
  public static function Dettach( EventHandlerI $handler ) {
    if ($position = array_search($handler, self::$EventHandlers, TRUE))
      unset(self::$EventHandlers[$position], self::$HandlerMethods[$position]);
    /// DebugStack::Info('Attached event handlers: '.count(self::$EventHandlers));
  }

  /**
   * Process all registered functions for an Event
   *
   * @public
   * @static
   * @param string $Event Event name
   */
  public static function Process( $Event, &$params ) {
    /// DebugStack::StartTimer($Event);

    $lEvent = strtolower($Event);
    if (!isset(self::$BlockedEvents[$lEvent])) {
      $Method = $Event . self::$Notification;
      $lMethod = strtolower($Method);
      foreach (self::$EventHandlers as $position=>$EventHandler) {
        if (isset(self::$HandlerMethods[$position][$lMethod])) {
          // >> Debug
          DebugStack::Info(get_class($EventHandler).'->'.$Method.'('.DebugStack::format($params).')');
          // << Debug
          $EventHandler->$Method($params);
        }
      }
    }
    // >> Debug
    else {
      DebugStack::Info('Skip "'.$Event.'", actually blocked.');
    }
    // << Debug
    /// DebugStack::StopTimer($Event);
  }

  /**
   * Process all registered functions for an event
   *
   * Don't manipulate the parameter and returns the result
   *
   * @public
   * @static
   * @param string $Event Event name
   */
  public static function ProcessReturn( $Event, $params=NULL ) {
    self::Process( $Event, $params );
    return $params[0];
  }

  /**
   * Process all registered handlers for an event
   *
   * For information about an event only, no further parameters required.
   * Mostly used for event 'Debug'
   *
   * @public
   * @static
   * @param string $Event Event name
   */
  public static function ProcessInform() {
    $params = func_get_args();
    // shift out Event
    $Event = array_shift($params);
    self::Process($Event, $params);
  }

  /**
   * Block an event, that should not executed (at the moment)
   *
   * @public
   * @static
   * @param string $Event Event name
   * @return void
   */
  public static function Block( $Event ) {
    self::$BlockedEvents[strtolower($Event)] = TRUE;
  }

  /**
   * Unblock an event
   *
   * @public
   * @static
   * @param string $Event Event name
   * @return void
   */
  public static function unBlock( $Event ) {
    $Event = strtolower($Event);
    if (isset(self::$BlockedEvents[$Event])) unset(self::$BlockedEvents[$Event]);
  }

  // -------------------------------------------------------------------------
  // PRIVATE
  // -------------------------------------------------------------------------

  /**
   * Array of handlers
   *
   * @var array
   */
  private static $EventHandlers = array();

  /**
   * Array of handler methods
   *
   * @var array
   */
  private static $HandlerMethods = array();

  /**
   * @var array
   */
  private static $BlockedEvents = array();

  /**
   * @var array
   */
  private static $Data;

}
