<?php
/**
 * Event handling
 *
 * @package Event
 */

/**
 * Events handling class
 */
abstract class Event implements EventI {

  // >> Debug
  /**
   * Shorten debugging parameter to this length
   *
   * @var int
   */
  public static $DBGLEN = 100;
  // << Debug

  /**
   *
   */
#  public static function getData() {
#    return self::$Data;
#  }

  /**
   *
   */
#  public static function setData( $data ) {
#    self::$Data = $data;
#  }

  /**
   * Naming of event handling methods: <Event>Notification()
   */
  public static $Notification = 'Notification';

  /**
   *
   */
  public static function Attach( EventHandlerI $handler, $position=0 ) {
    /// DebugStack::StartTimer(__METHOD__);
    // make sure not overwrite an existing handler, find next free position
    while (isset(self::$EventHandlers[$position])) $position++;

    // find handler methods
    $methods = array();
    $notification = strtolower(self::$Notification);
    foreach (get_class_methods($handler) as $method) {
      $method = strtolower($method);
      // can't be 0, is at the end of the method name ;-)
      if (strpos($method, $notification)) $methods[$method] = TRUE;
    }

    // Store only, if at least one possible handler method was found
    if (!empty($methods)) {
      self::$EventHandlers[$position] = $handler;
      ksort(self::$EventHandlers);
      self::$HandlerMethods[$position] = $methods;
    }

    /// DebugStack::Info('Attached event handlers: '.count(self::$EventHandlers));
    /// DebugStack::StopTimer(__METHOD__);
  }

  /**
   *
   */
  public static function Dettach( EventHandlerI $handler ) {
    if ($position = array_search($handler, self::$EventHandlers, TRUE))
      unset(self::$EventHandlers[$position], self::$HandlerMethods[$position]);
    /// DebugStack::Info('Attached event handlers: '.count(self::$EventHandlers));
  }

  /**
   * Process all registered functions for an Event
   *
   * @public
   * @static
   * @param string $event Event name
   * @param array $params Parameters for event notification
   */
  public static function Process( $event, &$params ) {
    /// DebugStack::StartTimer($event);
#    self::$Data = $params;

    $lEvent = strtolower($event);
    if (!isset(self::$BlockedEvents[$lEvent])) {
      $Method = $event . self::$Notification;
      $lMethod = strtolower($Method);
      foreach (self::$EventHandlers as $position=>$EventHandler) {
        if (isset(self::$HandlerMethods[$position][$lMethod])) {
          // >> Debug
          DebugStack::Info(get_class($EventHandler).'->'.$Method.'('.DebugStack::format($params).')');
          // << Debug
          $EventHandler->$Method($params);
#          $params = self::$Data;
        }
      }
    }
    // >> Debug
    else {
      DebugStack::Info('Skip "'.$event.'", actually blocked.');
    }
    // << Debug

    /// DebugStack::StopTimer($event);
  }

  /**
   * Process all registered functions for an event
   *
   * Don't manipulate the parameter and returns the result
   *
   * @public
   * @static
   * @param string $event Event name
   * @param array $params Parameters for event notification
   * @return mixed Using $params[0] as return value
   */
  public static function ProcessReturn( $event, $params=NULL ) {
    self::Process($event, $params);
    if (isset($params[0])) return $params[0];
  }

  /**
   * Process all registered handlers for an event
   *
   * For information about an event only, no further parameters required.
   * Mostly used for event 'Debug'
   *
   * @public
   * @static
   * @param string $event Event name
   * @return void
   */
  public static function ProcessInform() {
    $params = func_get_args();
    // shift out Event name
    $event = array_shift($params);
    self::Process($event, $params);
  }

  /**
   * Block an event, that should not executed (at the moment)
   *
   * @public
   * @static
   * @param string $event Event name
   * @return void
   */
  public static function Block( $event ) {
    self::$BlockedEvents[strtolower($event)] = TRUE;
  }

  /**
   * Unblock an event
   *
   * @public
   * @static
   * @param string $event Event name
   * @return void
   */
  public static function unBlock( $event ) {
    $event = strtolower($event);
    if (isset(self::$BlockedEvents[$event]))
		  unset(self::$BlockedEvents[$event]);
  }

  // -------------------------------------------------------------------------
  // PRIVATE
  // -------------------------------------------------------------------------

  /**
   * Array of handlers
   *
   * @var array
   */
  private static $EventHandlers = array();

  /**
   * Array of handler methods
   *
   * @var array
   */
  private static $HandlerMethods = array();

  /**
   * @var array
   */
  private static $BlockedEvents = array();

  /**
   * @var array
   */
  private static $Data;

}
