<?php
/**
 * serve.php - File serving script
 *
 * Usage:
 * <code>
 *   serve.php?base64_encode(<file name absolut or relative from script>)[&...]
 * </code>
 *
 * @package    serve
 * @author     Knut Kohl <knutkohl@users.sourceforge.net>
 * @copyright  2010 Knut Kohl
 * @version    1.0.0
 * @license    http://www.gnu.org/licenses/gpl.txt GNU General Public License
 */

/**
 * Define allowed file types (file extensions)
 */
// ---------------------------------------------------------------------------
$types = array(
// ---------------------------------------------------------------------------
// file ext.        content type      cache (compressed) file
  // styles
  'css'  => array( 'text/css',        TRUE  ),
  // scripts
  'js'   => array( 'text/javascript', TRUE  ),
  // images
  'gif'  => array( 'image/gif',       FALSE ),
  'jpg'  => array( 'image/jpeg',      FALSE ),
  'jpeg' => array( 'image/jpeg',      FALSE ),
  'png'  => array( 'image/png',       FALSE ),
  'tif'  => array( 'image/tiff',      FALSE ),
  'bmp'  => array( 'image/wbmp',      FALSE ),
// ---------------------------------------------------------------------------
);

/**
 * Cache directory
 */
#$CacheDir = '/tmp';
$CacheDir = dirname(__FILE__).'/../local/tmp';
$CacheDir = FALSE;

// ---------------------------------------------------------------------------
// Don't change from here
// ---------------------------------------------------------------------------

error_reporting(0);

// for testing
#error_reporting(-1);

// force re-read of file modification times
clearstatcache();

// use names for cache file name and whole array for ETag
$files = array( 'name' => array(), 'ts' => array() );

$type = NULL;

$args = explode('&', @$_SERVER['QUERY_STRING']);

$pretty = array_search('pretty', $args);
if ($pretty) unset($args[$pretty]);

foreach ((array)$args as $file) {
  $file1 = urldecode($file);
  if (file_exists($file1)) $file = $file1;
  elseif ($file1 = base64_decode($file)) $file = $file1;

#  if (substr($file, 0, 1) != DIRECTORY_SEPARATOR)
#    $file = dirname(__FILE__) . DIRECTORY_SEPARATOR . $file;

  if (!file_exists($file)) Error404('Sorry, file "'.$file.'" could not be found.');

#  $file = realpath($file);
  $ext = strtolower(substr($file, strrpos($file, '.')+1));

  if (!$type)
    $type = $ext;
  elseif ($ext != $type)
    Error404('Error: Only ONE file type per script run allowed!');

  $files['name'][] = $file;
  $files['ts'][] = filemtime($file);
}

if (empty($files['name'])) Error404('Error: No file given!');

$ETag = md5(serialize($files));

if (isset($_SERVER['HTTP_IF_NONE_MATCH']) AND strpos($_SERVER['HTTP_IF_NONE_MATCH'], $ETag))
  // Client's cache IS current, so we just respond '304 Not Modified'
  die(Header('HTTP/1.1 304 Not Modified'));

// not yet cached by client

// check type
if (isset($types[$type]))
  $ctype = $types[$type][0];
else
  Error404('File type "'.$type.'" is not allowed!');

// init. caching
$CacheFile = $content = '';

//  is $type cachable?
if ($types[$type][1] AND $CacheDir) {
  $CacheFile = $CacheDir . DIRECTORY_SEPARATOR . 'serve.' . md5(serialize($files['name'])) . '.' . $type;
  if (file_exists($CacheFile)) {
    $newer = FALSE;
    foreach ($files['name'] as $file) {
      if (filemtime($file) > filemtime($CacheFile)) {
        $newer = TRUE;
        break;
      }
    }
    if (!$newer) {
      // file is up-to-date
      Header('X-Cache: TRUE');
      $content = file_get_contents($CacheFile);
    }
  }
}

if (!$content) {
  // not cached

  $file = 'serve_'.$type.'.php';
  if (file_exists($file)) require_once $file;

  foreach ($files['name'] as $id=>$file) {
    $c = file_get_contents($file);
    $func = $type.'_pre_serve';
    if (function_exists($func)) $c = $func($c, $file);
    $content .= $c;
    $files['name'][$id] = str_replace($_SERVER['DOCUMENT_ROOT'].'/', '', realpath($file));
  }
  if ($content) {
    // compress
    $func = $type.'_post_serve';
    if (function_exists($func)) $content = $func($content, $files['name'], $pretty);
    // write cache?
    if ($CacheFile) file_put_contents($CacheFile, $content);
  }
}

// serve content
Header('ETag: "'.$ETag.'"');
Header('Cache-Control: must-revalidate, private');
Header('Content-Type: '.$ctype);
Header('Content-Length: '.strlen($content));
Header('Last-Modified: '.gmdate('D, d M Y H:i:s', time()).' GMT');
Header('HTTP/1.1 200 OK');
Header('Status: 200');

@ini_set('zlib.output_compression' , 'Off');

// zip content?
if (extension_loaded('zlib') AND strstr(@$_SERVER['HTTP_ACCEPT_ENCODING'], 'gzip')) {
  Header('Content-Encoding: gzip');
  $size = strlen($content);
  echo "\x1f\x8b\x08\x00\x00\x00\x00\x00",
       substr(gzcompress($content, 9), 0, -4),
       pack('V', crc32($content)),
       pack('V', $size);
} else {
  echo $content;
}

/**
 * Exit script with Header 404 Not Found
 *
 * @param string $msg Message to show
 */
function Error404( $msg ) {
  Header('Content-Type: text/plain');
  Header('HTTP/1.1 404 Not Found');
  die($msg);
}