<?php
/**
 * Compress JavaScript
 *
 * @package    serve
 * @author     Knut Kohl <knutkohl@users.sourceforge.net>
 * @copyright  2010 Knut Kohl
 * @version    1.0.0
 * @license    http://www.gnu.org/licenses/gpl.txt GNU General Public License
 *
 * @param string $content JavaScript to compress
 * @return string
 */
function js_post_serve( $content, $files, $pretty ) {

  $s1 = strlen($content);

  if (!$pretty) {
    // remove comments
    $content = preg_replace('~/\*.*?\*/~s', '', $content);
    $content = preg_replace('~^\s*//.*?$~m', '', $content);

    // compress
    $content = preg_replace('~[\r\n]*\s*([;{}])\s*[\r\n]*~', '$1', $content);

    // compress whitespaces
    $content = preg_replace('~\s+~', ' ', $content);
    $content = trim($content);

    // remove not required whitespaces outside output strings
    serve_js_strip( $content, '\'');
    serve_js_strip( $content, '"');
  }

  $s2 = strlen($content);

  return "/* " . implode("\n   ", $files) . "\n"
       . sprintf('   Compressed %d to %d Bytes (%.1f%%) */'."\n", $s1, $s2, $s2*100/$s1)
       . $content;
}

/**
 * @param string &$content JavaScript to compress
 * @param string $delimiter Delimiter to identify "not in a string" parts
 * @return void
 */
function serve_js_strip( &$content, $delimiter) {
  if (strpos($content, $delimiter) === FALSE) return;

  // split
  $parts = explode($delimiter, $content);
  $cnt = count($parts);
  $content = '';
  for ($i=0; $i<$cnt; $i++) {
    // compress "not in a string" parts
    $content .= preg_replace('~\s*(\W)\s*~', '$1', $parts[$i++]);
    // re-concatenate
    if (isset($parts[$i])) $content .= $delimiter . $parts[$i] . $delimiter;
  }
}