<?php
/**
 * @package es-f
 * @subpackage Setup
 * @desc |es|f| Setup
 */

// -----------------------------------------------------------------------------
# step definitions
// -----------------------------------------------------------------------------
$steps = array(
// step               Sub-Title           [, next step, next step on error]
  'intro'   => array('Welcome'                                              ),
  'cfg'     => array('Configuration'                                        ),
  'cfgchk'  => array(NULL                  , 'test'   , 'cfg'               ),
  'test'    => array('Permissions / Tests'                                  ),
  'user'    => array('Users'                                                ),
  'userchk' => array(NULL                  , 'save'   , 'user'              ),
  'save'    => array('Finished'                                             ),
);
// -----------------------------------------------------------------------------

error_reporting(0);
#error_reporting(E_ALL);

session_start();

define('_ESF_OK', TRUE);

define('BASEDIR', dirname(dirname(__FILE__)));

require_once '../application/define.php';

require_once APPDIR.'/classes/loader.class.php';
Loader::$AutoLoadPath[] = APPDIR.'/classes';

require_once APPDIR.'/init.php';
require_once APPDIR.'/functions.php';
require_once 'functions.php';

$AUTOLOAD_PATH[] = APPDIR.'/lib';

Cache::Init('Mock');

// Emulate register_globals off
unregister_GLOBALS();

$step = (isset($_REQUEST['step']) AND isset($_SESSION['CHECKED']))
      ? $_REQUEST['step'] : 'intro';
$step = isset($steps[$step]) ? $step : 'intro';
$stepdata = $steps[$step];
$Error = FALSE;

TplData::$NameSpaceSeparator = '.';

/**
 * pre process
 */
switch ($step) {

  // ------------
  case 'intro':
    // reset session data
    $_SESSION = array();
    // clear temp. directories
    exec('rm -rf ../local/data/tmp/*');
    exec('rm -rf ../local/data/session/*');

    if (!empty($_GET['msg'])) Messages::addError($_GET['msg']);

    LoadConfig();
    $_SESSION['USERS'] = Registry::get('Users');
    $_SESSION['INSTALL'] = (empty($_SESSION['USERS']) OR array_key_exists('DEVELOP', $_GET));
    $_SESSION['CHECKED']['intro'] = TRUE;
    break;

  // ------------
  case 'cfg':
    LoadConfig();

    $_SESSION['USERS'] = Registry::get('Users');
    // test for configuration versions below 5
    $cfg = empty($cfg) ? Registry::getAll() : array_merge(Registry::getAll(), $cfg);

    foreach ($cfg as $key => $val) TplData::set('cfg.'.$key, $val);

    $parsers = array();
    foreach (glob(APPDIR.'/classes/ebayparser/*.class.php') as $file)
      $parsers[] = basename($file, '.class.php');

    TplData::set('EBAYPARSERS', implode(', ', $parsers));

    TplData::set('LAYOUTS', getLayouts());
    exec('locale -a', $locales);
    TplData::set('LOCALES', $locales);
    unset($locales);

    if ($h = exec('which esniper')) TplData::set('cfg.bin_esniper', $h);

    foreach (Esniper::getAll() as $key => $val)
      TplData::set('esniper.'.strtoupper($key), $val);

    $ll = parse_ini_file(APPDIR.'/language/languages.ini');
    foreach (glob(APPDIR.'/language/*.php') as $l) {
      // check for not yet defined languages...
      $l = basename($l , '.php');
      if (!isset($ll[$l])) $ll[$l] = $l;
    }
    TplData::set('LANGUAGES', $ll);

    TplData::set('TIMEZONES', file(dirname(__FILE__).'/date.timezone', FILE_IGNORE_NEW_LINES));
    break;

  // ------------
  case 'cfgchk':
    savePosted('cfg');
    $_SESSION['CHECKED']['cfg'] = TRUE;
    break;

  // ------------
  case 'test':
    include 'step.test.php';
    break;

  // ------------
  case 'user':
    break;

  // ------------
  case 'userchk':
    checkPosted('remove', 'user', 'pass1', 'pass2');
    savePosted('user');
    $_SESSION['CHECKED']['user'] = TRUE;
    break;

  // ------------
  case 'save':
    include 'step.save.php';
    break;
}

if (!isset($stepdata[0])) {
  // Redirect
  session_write_close();
  Header('Location: index.php?step='.(!$Error ? $stepdata[1] : $stepdata[2]));
  Exit;
}

// -----------------------------------------------------------------------------
// Output
// -----------------------------------------------------------------------------
require_once 'config.php';

TplData::setConstant('ESF.TITLE', ESF_TITLE);
TplData::setConstant('ESF.SLOGAN', ESF_SLOGAN);
TplData::setConstant('ESF.VERSION', ESF_VERSION);
TplData::setConstant('ESF.RELEASE', ESF_RELEASE);
TplData::setConstant('ESF.HOMEPAGE', ESF_HOMEPAGE);
TplData::setConstant('ESF.LONG_TITLE', ESF_LONG_TITLE);
TplData::setConstant('ESF.FULL_VERSION', ESF_FULL_VERSION);
TplData::setConstant('ESF.FULL_TITLE', ESF_FULL_TITLE);
TplData::setConstant('BUTTON', '../button/button.php');
TplData::setConstant('IMAGES', urlencode('../setup/layout/default/images'));

TplData::setConstant('PHP_VERSION', PHP_VERSION);
TplData::setConstant('PHP_VERSION_REQUIRED', PHP_VERSION_REQUIRED);

TplData::set('USERS', $_SESSION['USERS']);
TplData::set('INSTALL', $_SESSION['INSTALL']);

TplData::set('SUBTITLE1', $stepdata[0]);
#TplData::set('SUBTITLE2', '');

// form data
TplData::merge(NULL, getSavedPosted($step));

$keys = array_keys($steps);
foreach ($keys as $id => $name) if ($step == $name) break;

TplData::set('BASEDIR', dirname(__FILE__));

TplData::set('FORM_IS_OPEN', FALSE);

TplData::set('NEXTSTEP', (isset($keys[$id+1]) ? $keys[$id+1] : ''));
TplData::set('NEXTTEXT', 'Next');

// skip redirect steps
while (isset($keys[$id-1]) AND !isset($steps[$keys[$id-1]][0])) $id--;
TplData::set('PREVSTEP', (isset($keys[$id-1]) ? $keys[$id-1] : ''));
TplData::set('PREVTEXT', 'Back');

if (version_compare(PHP_VERSION, PHP_VERSION_REQUIRED, '<')) $step = 'version';

Registry::set('TempDir', FALSE);

require_once dirname(__FILE__).'/../application/lib/yuelo/yuelo.require.php';

$Template = esf_Template::getInstance();

// Don't cache PHP code or static content
Yuelo::set('ReuseCode', FALSE);
Yuelo_Cache::Active(FALSE);

$RootDir = dirname(__FILE__).'/layout';

TplData::set('CONTENT', $Template->Render('step.'.$step, TRUE, $RootDir));
/**
 * post process
 */
switch ($step) {

  // ------------
  case 'cfg':
    TplData::set('FORM_IS_OPEN', TRUE);
    break;

  // ------------
  case 'test':
    if (TplData::get('ERROR')) {
      TplData::set('NEXTSTEP', 'test');
      TplData::set('NEXTTEXT', 'Reload');
    }
    break;

  // ------------
  case 'user':
    TplData::set('FORM_IS_OPEN', TRUE);
    TplData::set('NEXTTEXT', 'Save data');
    break;

  // ------------
  case 'save':
    TplData::set('PREVSTEP');
    break;

}

TplData::set('MESSAGES', (($msgs=Messages::get()) ? implode($msgs) : ''));

$Template->Output('index', TRUE, $RootDir);

// -----------------------------------------------------------------------------
// Finalize
// -----------------------------------------------------------------------------
switch ($step) {

  // ------------
  case 'save':
    // clear session data
    $_SESSION = array();
    break;
}
