#! /bin/bash
# ----------------------------------------------------------------------------
# /var/install/config.d/esniper - ESNIPER configuration
#
# Creation   : 2010-05-20 dr_snuggles
# Last update: $Id: esniper.sh 24179 2010-05-23 12:29:50Z dr_snuggles $
#
# Copyright (c) 2010 the eisfair team, <team(at)eisfair(dot)org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# ----------------------------------------------------------------------------

#exec 2> `pwd`/esniper-trace$$.log
#set -x

# ----------------------------------------------------------------------------
# Config Dateien
# ----------------------------------------------------------------------------

# include eislib
. /var/install/include/eislib

# include config
. /etc/config.d/esniper

# set package name
packageName=esniper

if [ -z "${EISFAIR_SYSTEM}" ]
then
    # get eisfair system-type
    . /var/install/include/check-eisfair-version
fi
# Setup system specific values
case "${EISFAIR_SYSTEM}" in
    eisfair-1)  # eisfair-1 code
        wwwuser="wwwrun:nogroup"
    ;;
    *)  # code for other systems
        wwwuser="www-data:www-data"
    ;;
esac

# ----------------------------------------------------------------------------
# define functions
# ----------------------------------------------------------------------------

backup_config()
{
     sh /var/install/bin/${packageName}-backup
}

restore_config()
{
    if [ -f "/usr/local/${packageName}/es-f_backup.tar.gz" ]
    then
        mecho ""
        reStore=`/var/install/bin/ask "Restore previous configuration?" "y"`
        if [ "${reStore}" == "yes" ]
        then
            sh /var/install/bin/${packageName}-restore
        fi
    fi
}

unpack_files()
{
    mecho "Unpacking esniper to ${ESNIPER_HTML_FOLDER}"
    if [ ! -d "${ESNIPER_HTML_FOLDER}" ]
    then
        mecho -warn "The path '${ESNIPER_HTML_FOLDER}' doesn't exist!"
        mecho -n    "Creating '${ESNIPER_HTML_FOLDER}'... "
        mkdir ${ESNIPER_HTML_FOLDER}
        mecho "Done"
    fi
    mecho -info "Unpacking..."
    tar -xjf /var/lib/${packageName}/esniper.tar.bz2 -C ${ESNIPER_HTML_FOLDER}
    chown -R ${wwwuser} ${ESNIPER_HTML_FOLDER}
    mecho -info "Done"
}

remove_esniper()
{
    if [ -f "${ESNIPER_HTML_FOLDER}/config.php" ]
    then
        backupconf=`/var/install/bin/ask "Backup the Config-Files?" "y"`
        if [ "${backupconf}" == "yes" ]
        then
            mecho ""
            backup_config
        fi
    fi
    mecho "Removing esniper..."
    rm -r ${ESNIPER_HTML_FOLDER}
    mecho "Done"
}

allow_url_fopen()
{
    if [ -f "/etc/php5/apache2/php.ini" ]
    then
        mecho ""
        mecho -n "Checking if 'allow_url_fopen' on php.ini is activated... "
        value=`cat /etc/php5/apache2/php.ini | grep "allow_url_fopen" | tr -s [[:space:]] | cut -d " " -f 3`
        case ${value} in
            'on'|'On'|'ON'|'oN')
                mecho "Done"
                ;;
            *)
                mecho ""
                mecho -warn "Option 'allow_url_fopen' is set to '${value}'!"
                activate=`/var/install/bin/ask "Set 'allow_url_fopen = On' in php.ini?" "y"`
                if [ "${activate}" == "yes" ]
                then
                    mecho -n "Activating option 'allow_url_fopen'... "
                    sed "s/allow_url_fopen.*/allow_url_fopen = On/g" /etc/php5/apache2/php.ini > /tmp/php.ini-$$
                    mv -f /tmp/php.ini-$$ /etc/php5/apache2/php.ini
                    mecho "Done"
                    . /etc/config.d/apache2
                    if [ "${START_APACHE2}" == "yes" ]
                    then
                        mecho "Restarting apache..."
                        /etc/init.d/apache2 restart
                        mecho "Done"
                    fi
                    mecho ""
                    mecho -info "Keep in mind that this setting is overwritten every time you re-/activate"
                    mecho -info "the PHP configuration via the setup menu!"
                    mecho ""
                fi
                ;;
        esac
    else
        mecho -warn "Could not check the setting 'allow_url_fopen' ('php.ini' not found). You"
        mecho -warn "have to check this by hand!"
    fi
}

# ----------------------------------------------------------------------------
# Main
# ----------------------------------------------------------------------------
if [ "${START_ESNIPER}" == "yes" ]
then
    if [ -d "${ESNIPER_HTML_FOLDER}" ]
    then
        mecho ""
        reInstall=`/var/install/bin/ask "Delete old and reinstall esniper files?" "n"`
        if [ "${reInstall}" == "yes" ]
        then
            remove_esniper
            unpack_files
            allow_url_fopen
            restore_config
        else
            allow_url_fopen
            restore_config
        fi
    else
        unpack_files
        allow_url_fopen
        restore_config
    fi
else
    if [ -d "${ESNIPER_HTML_FOLDER}" ]
    then
        remove_esniper
    fi
fi

exit 0
# ----------------------------------------------------------------------------
# End
# ----------------------------------------------------------------------------
